#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <math.h>
#include <io.h>
#include <dos.h>
#include <conio.h>

#define FALSE         0
#define TRUE  (! FALSE)
#define MAXSIZE      11

/* code will handle up to a MAXSIZE-1 array */

struct matrix {double M[MAXSIZE];};

double mu,mustar;

/* ------------------------------------------------------------------ */

double sqr(double invalue)
{
  return (invalue*invalue);
}

double cube(double invalue)
{
  return (invalue*invalue*invalue);
}

/* ------------------------------------------------------------------ */

void fvect(struct matrix * X,     /* matrix in */
			  struct matrix * F)     /* matrix out */
{
  double r1,r2;

  r1 = sqrt( sqr( X->M[2] + mu ) + sqr( X->M[3] ) );
  r2 = sqrt( sqr( X->M[2] - mustar) + sqr( X->M[3]) );

  F->M[1] = 1.0;
  F->M[2] = X->M[4];
  F->M[3] = X->M[5];
  F->M[4] = ( 2.0*X->M[5] ) +
				( X->M[2] ) -
				( (mustar * (X->M[2] + mu))/cube(r1) ) -
				( (mu * (X->M[2] - mustar))/cube(r2));
  F->M[5] = ( -2.0 * X->M[4] ) +
				( X->M[3] ) -
				( (mustar * X->M[3]) / cube(r1) ) -
				( (mu * X->M[3]) / cube(r2));
}

/* ------------------------------------------------------------------ */

void rk(int n,              /* array size */
		  struct matrix * X,  /* X array to work with */
		  double h,           /* step size */
		  int nstep)          /* number of steps */
{
  int i,j;
  struct matrix y,f1,f2,f3,f4;

  for (j=1;j<=nstep;j++)
  {
	  fvect(X,&f1);

	  for (i=1;i<=n;i++)
		 y.M[i]= X->M[i] + 0.5*h*f1.M[i];
	  fvect(&y,&f2);

	  for (i=1;i<=n;i++)
		 y.M[i] = X->M[i] +0.5*h*f2.M[i];
	  fvect(&y,&f3);

	  for (i=1;i<=n;i++)
		 y.M[i] = X->M[i] + h*f3.M[i];
	  fvect(&y,&f4);

	  for (i=1;i<=n;i++)
		 X->M[i] = X->M[i] + ( f1.M[i] + 2.0*f2.M[i] + 2.0*f3.M[i] + f4.M[i] ) * (h/6.0);
  };
}

/* ----------------------------------------------------------------- */
void main(argc,argv)
  int argc;
  char *argv[];
{
  struct matrix BIGX;
  FILE *output;
  int i,mintime,minclos;

  double currentmag,   /* current magnitude to Earth's surface */
			minmag,       /* minimum magnitude (x,y) to Earth's surface */
			currentclos,  /* current magnitude to original stating point */
			closmag;      /* closest magnitude (x,y) to original starting point */

  /* is used closest because the full 1 orbit value may not be exactly the same
	  as the original starting point;
	  program will start checking for this value after 1 time unit so that
	  the first iteration value (t=.001 time units from start) does not become closmag */

  minmag  = 10.0; /* starting value set bigger than known starting point of minmag = 1.2 */
  closmag = 10.0;

  mu = 1.0/82.45;
  mustar = 1.0 - mu;


  output=fopen("threebod.txt","w");

  BIGX.M[1] = 0.0;
  BIGX.M[2] = 1.2;
  BIGX.M[3] = 0.0;
  BIGX.M[4] = 0.0;
  BIGX.M[5] = -1.04935751;

  fprintf(output," **** MEG 777 Three Body Problem in C ****\n\n");

  fprintf(output,"step value = 0.001 time units\n");
  fprintf(output,"values printed out every ten 0.001 time units =\n");
  fprintf(output,"      4.35 * 10 * 0.001 days = .0435 days = 62.64 minutes\n\n");

  fprintf(output," i              x              y\n");

  for (i=1;i<=10000;i++)
  {
	 rk(5,&BIGX,0.001,1);       /* slow, but check every single value */

	 currentmag = sqrt(sqr(BIGX.M[2]+mu) + sqr(BIGX.M[3]));
	 if (currentmag < minmag)
	 {
		minmag = currentmag;
		mintime = i;
	 }

	 if (i>1000)    /* only check for complete orbit after 1 (normalized) time unit */
	 {
		currentclos = sqrt(sqr(BIGX.M[2] - 1.2) + sqr(BIGX.M[3] - 0.0));
		if (currentclos < closmag)
		{
		  closmag = currentclos;
		  minclos = i;
		}
	 }

	 if ((i%10) == 0)
	 {
		printf("at i = %d\n",i);
		fprintf(output,"%5d   %15.5f   %15.5f\n",i,BIGX.M[2],BIGX.M[3]);
	 }
  }

  fprintf(output,"Closest approach to Earth (perigee):\n");
  fprintf(output,"===========================\n");
  fprintf(output,"normalized time (1 time unit = 4.35 days) : %15.5f\n",mintime/1000.0);
  fprintf(output,"real time : %15.5f days\n",(mintime/1000.0) * 4.35);
  fprintf(output,"Distance to center of Earth at perigee:\n");
  fprintf(output,"---------------------------------------\n");
  fprintf(output,"normalized distance (1 distance unit = 238,000 miles) : %15.5f\n",minmag);
  fprintf(output,"real distance : %15.5f miles\n",minmag*238000.0);
  fprintf(output,"Distance to surface of Earth (R=4000 miles) at perigee:\n");
  fprintf(output,"-------------------------------------------------------\n");
  fprintf(output,"real distance : %15.5f miles\n",(minmag*238000.0)-4000.0);

  fprintf(output,"\n\n\n");

  fprintf(output,"Closest approach to original position:\n");
  fprintf(output,"======================================\n");
  fprintf(output,"normalized time (1 time unit = 4.35 days) : %15.5f\n",minclos/1000.0);
  fprintf(output,"real time : %15.5f days\n",(minclos/1000.0) * 4.35);
  fprintf(output,"Distance to original position at this time:\n");
  fprintf(output,"-------------------------------------------\n");
  fprintf(output,"normalized distance (1 distance unit = 238,000 miles) : %15.5f\n",closmag);
  fprintf(output,"real distance : %15.5f miles\n",closmag*238000.0);

  fclose(output);

  printf("Program done.\n");
  getc(stdin);
}


